/**
 * Handles reality check pauses in a way that won't interrupt gameplay. If a pause is triggered mid-round,
 * the game is only paused after that round ends.
 */
export interface IPausePayload {
    id?: string;
    realityCheck?: boolean;
    autoPlayOnly?: boolean;
}
/**
 * In some situations reality check is triggered by an internal timer in FEIM, not the operator postMessages.
 * This will create that timer if the environment requires it.
 * This is only used by the Legacy RC.
 */
export declare const handleClientSideRcInterval: () => void;
/**
 * @internal
 * Internal handler for reality check or other pause messages.
 * The game will be frozen once a round is no longer in progress, and a paused event is dispatched.
 */
export declare const handleIdlePause: () => void;
