import { IErrorMessage } from "../feim/types";
/**
 * An async alternative to send.errorMessage. Shows an error popup - using the remote error message system if used by the operator -
 * and resolves once the error popup is closed. Requires [[showErrorMessageCallback]] to be configured.
 *
 * Will trigger on.[[freeze]] when the error is shown, and on.[[unfreeze]] when the error is resolved.
 *
 * If the operator _is not_ handling error popups, the given [[showErrorMessageCallback]] is called. If [[showErrorMessageCallback]] returns a Promise and
 * it is resolved, the Promise returned by this function is resolved as well.
 * If [[showErrorMessageCallback]] does not return a Promise, the Promise returned by this function is resolved immediately.
 *
 * If the operator _is_ handling error popups, the module will send the error message data to the operator.
 * The promise returned by this function is resolved once that error message is dismissed in the operator's window.
 *
 * This is a convenience function, using send.errorMessage, on.errorMessageDisplayed and on.errorMessageDismissed internally.
 *
 * ```
 * await showError(casinoResponseContainingError)
 * console.log("Error message closed, continue game")
 * ```
 *
 * @param message - The casino response containing the error
 *
 * @returns A Promise which will be resolved when the operator popup is closed, or, if the operator isn't handling error popups, when the [[showErrorMessageCallback]] Promise is resolved.
 */
export declare const showError: (message: IErrorMessage) => Promise<void>;
